@@ -1,6 +1,6 @@
 package Test::DBIx::Class::Schema;
 BEGIN {
-  $Test::DBIx::Class::Schema::VERSION = '0.01010';
+  $Test::DBIx::Class::Schema::VERSION = '0.01011';
 }
 BEGIN {
   $Test::DBIx::Class::Schema::DIST = 'Test-DBIx-Class-Schema';
@@ -30,7 +30,7 @@ sub methods {
 
 sub run_tests {
     my ($self) = @_;
-    my ($schema, $record, $resultset);
+    my ($schema, $rs, $record);
 
     # make sure we can use the schema (namespace) module
     use_ok( $self->{namespace} );
@@ -51,83 +51,116 @@ sub run_tests {
 
     # create a new resultset object and perform tests on it
     # - this allows us to test ->my_column() without requiring data
-    my $rs = $schema->resultset( $self->{moniker} )->new({});
+    $rs = $schema->resultset( $self->{moniker} );
+    $record = $schema->resultset( $self->{moniker} )->new({});
+
+    # make sure our record presents itself as the correct object type
+    if (defined $self->{glue}) {
+        isa_ok(
+            $record,
+                $self->{namespace}
+            . '::' . $self->{glue}
+            . '::' . $self->{moniker}
+        );
+    }
+    else {
+        isa_ok($record, $self->{namespace} . '::' . $self->{moniker});
+    }
+
     $self->_test_normal_methods($rs);
-    $self->_test_special_methods($rs);
+    $self->_test_special_methods($record);
     $self->_test_resultset_methods($rs);
 
-    # get an object to test methods against
-    #   changed from ->first() to ->slice()->single() at dakkar's reqest
-    #   he say's it's faster
-    $record = $schema->resultset( $self->{moniker} )
-                ->search({})
-                ->slice(0,0)
-                ->single();
-
-    # no actual records - don't test against nothingness
-    SKIP: {
-        skip q{no records in the table}, 1
-            if (not defined $record);
-
-        # run tests on real records
-        if (defined $self->{glue}) {
-            isa_ok(
-                $record,
-                  $self->{namespace}
-                . '::' . $self->{glue}
-                . '::' . $self->{moniker}
-            );
-        }
-        else {
-            isa_ok($record, $self->{namespace} . '::' . $self->{moniker});
-        }
-        eval {
-            $schema->txn_do(
-                sub {
-                    # 'normal' methods; row & relation
-                    # we can try calling these as they gave no side-effects
-                    $self->_test_normal_methods($record);
-                    $self->_test_special_methods($record);
-                    $self->_test_resultset_methods($record);
-
-
-                    # rollback any evil changes that crept through from the
-                    # tested calls
-                    $schema->txn_rollback;
-                }
-            )
-        };
-        if (my $e=$@) {
-            warn $e;
-        }
-    }; # end SKIP
-
     done_testing
         unless $ENV{TEST_AGGREGATE};
 }
 
 sub _test_normal_methods {
     my $self    = shift;
-    my $record  = shift;
+    my $rs  = shift;
 
     my @std_method_types        = qw(columns relations);
 
     # 'normal' methods; row & relation
     # we can try calling these as they gave no side-effects
+    my @proxied;
     foreach my $method_type (@std_method_types) {
         SKIP: {
             if (not @{ $self->{methods}->{$method_type} }) {
                 skip qq{no $method_type methods}, 1;
             }
 
-            can_ok(
-                $record,
-                @{ $self->{methods}->{$method_type} },
-            );
             # try calling each method
             foreach my $method ( @{ $self->{methods}->{$method_type} } ) {
-                eval { $record->$method };
-                is($@, q{}, qq{calling $method() didn't barf});
+                # make sure we can call the method
+                my $source = $rs->result_source;
+
+                # 'normal' relationship
+                if ($source->has_relationship($method)) {
+                    eval {
+                        my $related_source = $source->related_source($method);
+                    };
+                    is($@, q{}, qq{related source for '$method' OK});
+
+                    next; # skip the tests that don't apply (below)
+                }
+
+                # many_to_many and proxy
+                if ( $method_type eq 'relations' ) {
+                    $DB::single=1 if $method eq 'currency';
+                    my $result = $rs->new({});
+                    if (can_ok( $result, $method )) {
+                        my @relationships = $source->relationships;
+                        my $is_proxied;
+                        for my $relationship ( @relationships ) {
+                            my $proxy =
+                                $source->relationship_info($relationship)->{attrs}{proxy};
+                            # If the relationship is proxied then we assume it
+                            # works if we can call it, and it should be tested
+                            # in the related result source
+                            next if not $proxy;
+                            $is_proxied = 1;
+                            pass qq{'$method' relationship exists via proxied relationship '$relationship'};
+                            last;
+                        }
+                        # many_to_many
+                        isa_ok( $result->$method, 'DBIx::Class::ResultSet')
+                            if not $is_proxied;
+                    }
+                }
+
+                # column accessor
+                elsif ( $method_type eq 'columns' ) {
+                    if ( $source->has_column($method) ) {
+                        pass qq{'$method' column defined in result_source};
+                        eval {
+                            my $col = $rs->get_column($method)->all;
+                        };
+                        is($@, q{}, qq{'$method' column exists in database});
+                    }
+                    else {
+                        my @relationships = $source->relationships;
+                        for my $relationship ( @relationships ) {
+                            my $proxy =
+                                $source->relationship_info($relationship)->{attrs}{proxy};
+                            next if not $proxy;
+                            if ( grep m{$method}, @$proxy ) {
+                                eval { $rs->new({})->$method; };
+                                is($@, q{}, qq{'$method' column exists via proxied relationship '$relationship'});
+                            }
+                            else {
+                                fail qq{'$method' column does not exist and is not proxied};
+                            }
+                            last;
+                        }
+                    }
+                    #ok($source->has_column($method), qq{$method: column defined in result_source});
+                }
+
+                # ... erm ... what's this?
+                else {
+                    die qq{unknown method type: $method_type};
+                }
             }
         }
     } # foreach
@@ -135,75 +168,51 @@ sub _test_normal_methods {
 }
 
 sub _test_special_methods {
-    my $self    = shift;
-    my $record  = shift;
+    shift->_test_methods(shift, [qw/custom/]);
+}
 
-    my @special_method_types    = qw(custom);
+sub _test_resultset_methods {
+    shift->_test_methods(shift, [qw/resultsets/]);
+}
+
+sub _test_methods {
+    my $self            = shift;
+    my $thingy          = shift;
+    my $method_types    = shift;
 
     # 'special' methods; custom
     # we can't call these as they may have unknown parameters,
     # side effects, etc
-    foreach my $method_type (@special_method_types) {
+    foreach my $method_type (@{ $method_types} ) {
         SKIP: {
-            if (not @{ $self->{methods}->{$method_type} }) {
-                skip qq{no $method_type methods}, 1;
-            }
-
-            can_ok(
-                $record,
-                @{ $self->{methods}->{$method_type} },
+            skip qq{no $method_type methods}, 1
+                    unless @{ $self->{methods}{$method_type} };
+            ok(
+                @{ $self->{methods}{$method_type} },
+                qq{$method_type list found for testing}
             );
         }
+
+        # call can on each method to make it obvious what's being tested
+        foreach my $method (@{ $self->{methods}{$method_type} } ) {
+            can_ok( $thingy, $method );
+        }
     } # foreach
     return;
 }
 
-sub _test_resultset_methods {
-    my $self        = shift;
-    my $schema      = shift->result_source->schema;
-    my $resultset   = $schema->resultset( $self->{moniker} );
-
-    my @resultset_method_types  = qw(resultsets);
-
-    # resultset class methods - we need something slightly different here
-    foreach my $method_type (@resultset_method_types) {
-        SKIP: {
-            skip qq{no resultsets methods}, 1
-                unless @{ $self->{methods}->{resultsets} };
-
-            can_ok(
-                $resultset,
-                @{ $self->{methods}->{resultsets} },
-            );
-        } # SKIP, no resultsets
-    } # foreach
-    return;
-}
-
 1;
-__END__
+
 
 =pod
 
 =head1 NAME
 
-Test::DBIx::Class::Schema - DBIx::Class schema sanity checking tests
+Test::DBIx::Class::Schema
 
 =head1 VERSION
 
-version 0.01010
-
-=head1 DESCRIPTION
-
-It's really useful to be able to test and confirm that DBIC classes have and
-support a known set of methods.
-
-Testing these one-by-one is more than tedious and likely to discourage you
-from writing the relevant test scripts.
-
-As a lazy person myself I don't want to write numerous near-identical scripts.
-
-Test::DBIx::Class::Schema takes the copy-and-paste out of DBIC schema class testing.
+version 0.01011
 
 =head1 SYNOPSIS
 
@@ -281,6 +290,22 @@ If you are running aggregated tests you will need to add
 
 to your top-level script.
 
+=head1 DESCRIPTION
+
+It's really useful to be able to test and confirm that DBIC classes have and
+support a known set of methods.
+
+Testing these one-by-one is more than tedious and likely to discourage you
+from writing the relevant test scripts.
+
+As a lazy person myself I don't want to write numerous near-identical scripts.
+
+Test::DBIx::Class::Schema takes the copy-and-paste out of DBIC schema class testing.
+
+=head1 NAME
+
+Test::DBIx::Class::Schema - DBIx::Class schema sanity checking tests
+
 =head1 SEE ALSO
 
 L<DBIx::Class>,
@@ -293,15 +318,31 @@ Chisel Wright C<< <chisel@chizography.net> >>
 
 =head1 CONTRIBUTORS
 
-Gianni Ceccarelli C<< <dakkar@thenautilus.net> >>
+Gianni Ceccarelli C<< <dakkar@thenautilus.net> >>,
+Darius Jokilehto
 
 =head1 LICENSE
 
-Copyright 2008-2010 by Chisel Wright
+Copyright 2008-2011 by Chisel Wright
 
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 See <http://www.perl.com/perl/misc/Artistic.html>
 
-=cut
\ No newline at end of file
+=head1 AUTHOR
+
+Chisel Wright <chisel@chizography.net>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2011 by Chisel Wright.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
+
+
+__END__
+
